@@ -1,15 +1,21 @@
 """ Utilities for fedbadges that don't quite fit anywhere else. """
 
+import json
 import types
 
 import logging
 log = logging.getLogger("moksha.hub")
 
 import fedmsg
+import requests
+
+# This is used for our queries against pkgdb
+from dogpile.cache import make_region
+_cache = make_region()
+
 
 # These are here just so they're available in globals()
 # for compiling lambda expressions
-import json
 import re
 import fedmsg.config
 import fedmsg.encoding
@@ -48,6 +54,8 @@ def format_args(obj, subs):
         return [format_args(item, subs) for item in obj]
     elif isinstance(obj, basestring) and obj[2:-2] in subs:
         obj = subs[obj[2:-2]]
+    elif isinstance(obj, (int, float)):
+        pass
     else:
         obj = obj % subs
 
@@ -107,3 +115,57 @@ def notification_callback(topic, msg):
         topic=topic,
         msg=msg,
     )
+
+
+def get_pkgdb_packages_for(config, username):
+    """ Retrieve the list of packages where the specified user some acl.
+
+    :arg config: a dict containing the fedmsg config
+    :arg username: the fas username of the packager whose packages are of
+        interest.
+    :return: a set listing all the packages where the specified user has
+        some ACL.
+
+    """
+
+    if not hasattr(_cache, 'backend'):
+        _cache.configure(**config['fedbadges.rules.cache'])
+
+    @_cache.cache_on_arguments()
+    def _getter(username):
+        if config.get('fedbadges.rules.utils.use_pkgdb2', True):
+            return _get_pkgdb2_packages_for(config, username)
+        else:
+            return _get_pkgdb1_packages_for(config, username)
+
+    return _getter(username)
+
+
+def _get_pkgdb2_packages_for(config, username):
+    log.debug("Requesting pkgdb2 packages for user %r" % username)
+    req = requests.get('{0}/packager/acl/{1}'.format(
+        config['fedbadges.rules.utils.pkgdb_url'], username))
+    if not req.status_code == 200:
+        return set()
+    data = json.loads(req.text)
+    packages = set()
+    for pkgacl in data['acls']:
+        if pkgacl['status'] != 'Approved':
+            continue
+        packages.add(pkgacl['packagelist']['package']['name'])
+    log.debug("done talking with pkgdb2 for now.")
+    return packages
+
+
+# TODO -- delete this once pkgdb2 goes live.
+def _get_pkgdb1_packages_for(config, username):
+    log.debug("Requesting pkgdb1 packages for user %r" % username)
+    pkgdb1_base_url = config['fedbadges.rules.utils.pkgdb_url']
+    req = requests.get('{0}/users/packages/{1}?tg_format=json'.format(
+        pkgdb1_base_url, username))
+    if not req.status_code == 200:
+        return set()
+    data = json.loads(req.text)
+    packages = set([pkg['name'] for pkg in data['pkgs']])
+    log.debug("done talking with pkgdb1 for now.")
+    return packages
